# PasFacer.py - Updates pascal units used to create an object pascal scintilla wrapper.

import string
import sys
import os
import PFace

def Contains(s,sub):
	return string.find(s, sub) != -1

def pType(s):
	if s == "bool":
		return "LongBool"
	elif s == "position":
		return "LongInt"
	elif s == "colour":
		return "LongInt"
	elif s == "string":
		return "PChar"
	elif s == "stringresult":
		return "PChar"
	elif s == "cells":
		return "PChar"
	elif s == "textrange":
		return "PTextRange"
	elif s == "findtext":
		return "PTextToFind"
	elif s == "keymod":
		return "LongInt"
	elif s == "formatrange":
		return "LongInt"
	elif s == "int":
		return "LongInt"
	return ""

def fixConstant(c):
	r = string.replace(c, "|", "or")
	r = string.replace(r, "0x", "$")
	return r

def printDefines(f, out):
	for name in f.order:
		v = f.features[name]
		if v["Category"] != "Deprecated":
			if v["FeatureType"] in ["fun", "get", "set"]:
				featureDefineName = "SCI_" + string.upper(name)
				out.write("    " + featureDefineName + " = " + fixConstant(v["Value"]) + ";\n")
			elif v["FeatureType"] in ["evt"]:
				featureDefineName = "SCN_" + string.upper(name)
				out.write("    " + featureDefineName + " = " + fixConstant(v["Value"]) + ";\n")
			elif v["FeatureType"] in ["val"]:
				featureDefineName = string.upper(name)
				out.write("    " + featureDefineName + " = " + fixConstant(v["Value"]) + ";\n")

def genEventPrototype(name, evt):
	retval = string.lower(name) + " = procedure(Sender : TObject"
	for parm in evt["Params"].order:
		v = evt["Params"].params[parm]
		retval = retval + "; " + parm + " : " + pType(v["ParamType"])
	retval = retval + ")"
	return retval

def printEventDefs(f, out):
	for name in f.order:
		v = f.features[name]
		if v["Category"] != "Deprecated":
			if v["FeatureType"] in ["evt"]:
				eventDefineName = "TSCEvent_" + genEventPrototype(name, v)
				eventDefineName = eventDefineName + " of object;\n"
				out.write("    " + eventDefineName)

def printEventPrivates(f, out):
	for name in f.order:
		v = f.features[name]
		if v["Category"] != "Deprecated":
			if v["FeatureType"] in ["evt"]:
				eventtname = "TSCEvent_" + string.lower(name) + ";\n"
				eventvname = "    FOn" + string.lower(name) + " : " + eventtname;
				out.write("    " + eventvname)

def printEventProperties(f, out):
	for name in f.order:
		v = f.features[name]
		if v["Category"] != "Deprecated":
			if v["FeatureType"] in ["evt"]:
				#property OnStyleNeeded : TSCEvent_styleneeded read FOnstyleneeded write FOnstyleneeded;
				prop = "FOn" + string.lower(name)
				line = "property On" + name + " : TSCEvent_" + string.lower(name)
				line = line + " read " + prop + " write " + prop + ";\n"
				out.write("    " + line)

def printEventImpl(f, out):
	for name in f.order:
		v = f.features[name]
		if v["Category"] != "Deprecated":
			if v["FeatureType"] in ["evt"]:
				#2000 : if assigned(FOnstyleneeded) then FOnstyleneeded(Self, scn^.position);
				line = "      " + v["Value"] + " : if assigned("
				line = line + "FOn" + string.lower(name) + ") then FOn" + string.lower(name) + "(Self"
				for parm in v["Params"].order:
					z = v["Params"].params[parm]
					line = line + ", scn^." + parm
				line = line + ");\n"
				out.write(line)
				

def genfuncPrototype(name, fun):
	retval = name
	badded = 0
	if fun["Param1Type"] != "" and fun["Param1Type"] != " ":
		badded = 1
		retval = retval + "(" + fun["Param1Name"] + " : " + pType(fun["Param1Type"])
	if fun["Param2Type"] != "" and fun["Param2Type"] != " ":
		if badded == 1:
			retval = retval + "; "
		else:
			retval = retval + "("
		badded = 1
		retval = retval + fun["Param2Name"] + " : " + pType(fun["Param2Type"])
	if badded == 1:
		retval = retval + ")"
	return retval

def getfuncHeader(name, fun, extra):
	retval = genfuncPrototype(name, fun)
	line = ""
	if pType(fun["ReturnType"]) == "":
		line = line + "procedure " + extra + retval + ";\n"
	else:
		line = line + "function "  + extra + retval + " : " + pType(fun["ReturnType"]) + ";\n"
	return line

def printFunctionDefs(f, out):
	for name in f.order:
		v = f.features[name]
		if v["Category"] != "Deprecated":
			if v["FeatureType"] in ["fun", "set", "get"]:
				line = "    "
				line = line + getfuncHeader(name, v, "")
				out.write(line)

def isFunction(v):
	if pType(v["ReturnType"]) == "":
		ret = 0
	else:
		ret = 1
	return ret

def printFunctionImpl(f, out):
	for name in f.order:
		v = f.features[name]
		if v["Category"] != "Deprecated":
			if v["FeatureType"] in ["fun", "set", "get"]:
				header = getfuncHeader(name, v, "TScintilla.")
				out.write(header)
				out.write("begin\n")
				line = "  "
				endline = ""
				if not isFunction(v):
					line = line + "SCPerform(sccmdctr, "
				else:
					line = line + "result := "
					if pType(v["ReturnType"]) != "LongInt":
						line = line + pType(v["ReturnType"]) + "("
						endline = ")"
					line = line + "SCPerform(sccmdctr, "
				line = line + "SCI_" + string.upper(name) + ", "
				if not (v["Param1Type"] in ["", " "]):
					add = ""
					if pType(v["Param1Type"]) != "LongInt":
						line = line + "LongInt("
						add = ")"
					line= line + v["Param1Name"] + add + ", "
				else:
					line = line + "0, "
				if not (v["Param2Type"] in ["", " "]):
					add = ""
					if pType(v["Param2Type"]) != "LongInt":
						line = line + "LongInt("
						add = ")"
					line = line + v["Param2Name"] + add + ")"
				else:
					line = line + "0)"
				line = line + endline + ";"
				out.write(line + "\nend;\n\n")

def genMainControl(input, output, definition):
	copying = 1
	for line in input.readlines():
		if copying:
			output.write(line)
		#if Contains(line, "//++Autogenerated"):
		#	copying = 0
		#	genfn(definition, output)
		if Contains(line, "//++EventTypes"):
			copying = 0
			printEventDefs(definition, output)
		if Contains(line,"//++EventPrivates"):
			copying = 0
			printEventPrivates(definition, output)
		if Contains(line, "//++EventProperties"):
			copying = 0
			printEventProperties(definition, output)
		if Contains(line,"//++FuncDef"):
			copying = 0
			printFunctionDefs(definition, output)
		if Contains(line,"//++Const"):
			copying = 0
			printDefines(definition, output)
		if Contains(line,"//++FuncImp"):
			copying = 0
			printFunctionImpl(definition, output)
		if Contains(line,"//++EventImpl"):
			copying = 0
			printEventImpl(definition, output)
		if Contains(line,"//--"):
			copying = 1
			output.write(line)

def genConsts(input, output, definition):
	copying = 1
	for line in input.readlines():
		if copying:
			output.write(line)
		if Contains(line,"//++Const"):
			copying = 0
			printDefines(definition, output)
		if Contains(line,"//--") and copying == 0:
			copying = 1
			output.write(line)

def Regenerate(filename, outputfilename, definition, fn):
	tempname = "PasFacer.tmp"
	out = open(tempname,"w")
	hfile = open(filename)
	#CopyWithInsertion(hfile, out, definition)
	fn(hfile, out, definition)
	out.close()
	hfile.close()
	if(os.access(outputfilename, os.F_OK)):
		os.unlink(outputfilename)
	os.rename(tempname, outputfilename)

# Program Start
f = PFace.Face()
f.ReadFromFile("..\\include\\Scintilla.iface")
Regenerate("..\\originals\\SciLexer.pas", "..\\pascal\\SciLexer.pas", f, genMainControl)
Regenerate("..\\originals\\SciSupport.pas", "..\\pascal\\SciSupport.pas", f, genConsts)